//[*]--------------------------------------------------------------------------------------------------[*]
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <android/log.h>
#include <sys/ioctl.h>

//[*]--------------------------------------------------------------------------------------------------[*]
#include "Scope.h"

//[*]--------------------------------------------------------------------------------------------------[*]
#define LOG_TAG     "ADC"

//#define LOGI(...)
#define LOGI(...)  __android_log_print(ANDROID_LOG_INFO, LOG_TAG, __VA_ARGS__)
//[*]--------------------------------------------------------------------------------------------------[*]
#define PERIODS_COUNT      10 

const unsigned int PERIODS[] = {
	PERIOD_6000us,
	PERIOD_5400us,
	PERIOD_4800us,
	PERIOD_4200us,
	PERIOD_3600us,
	PERIOD_3000us,
	PERIOD_2400us,
	PERIOD_1800us,
	PERIOD_1200us,
	PERIOD_600us,
};

//[*]--------------------------------------------------------------------------------------------------[*]
#define VOLT_DIV_COUNT      9

const float VOLT_PER_DIV[] = {
	SCALE_GND,
	SCALE_2V,
	SCALE_1V,
	SCALE_500mV,
	SCALE_200mV,
	SCALE_100mV,
	SCALE_50mV,
	SCALE_20mV,
	SCALE_10mV
};

//[*]--------------------------------------------------------------------------------------------------[*]
//
// Global variable
//
//[*]--------------------------------------------------------------------------------------------------[*]
volatile                int    gFd      = -1;
volatile    unsigned    int    gScale   = 0;
volatile    unsigned    int    gPeriod  = 0;
volatile    unsigned    int    gDataPool[DRIVER_MAX_SAMPLE];

//[*]--------------------------------------------------------------------------------------------------[*]
//
// Function Prototype define
//
//[*]--------------------------------------------------------------------------------------------------[*]
int             OpenDevice      (void);
void            CloseDevice     (void);
int             OpenChannel     (void);
int             StopChannel     (void);
void            SetScale        (unsigned int scale);
int             GetScale        (void);
float           GetVoltScale    (void);
void            SetPeriod       (unsigned int period);
unsigned int    GetPeriod       (void);
unsigned int    GetDataSize     (void);
int             GetDeviceData   (unsigned int scale, unsigned int period);
int             GetPlotData     (unsigned int *pData);

//[*]--------------------------------------------------------------------------------------------------[*]
//
// Application   ῡ  ó
//
//[*]--------------------------------------------------------------------------------------------------[*]
int             OpenDevice      (void)
{
	LOGI("gFd = %d\n");
        
    gFd = open(ADC_DEVICE_NODE, O_RDWR | O_NOCTTY | O_NONBLOCK, 0644);
	LOGI("gFd = %d\n");

    if(gFd < 0)     {
        LOGI("%s : Open failed! (gFd < 0) Error!", __FUNCTION__);
        return  -1;     // Can't Opend
    }
    
    LOGI("%s : %s opened!", __FUNCTION__, ADC_DEVICE_NODE);

    return  0;
}

//[*]--------------------------------------------------------------------------------------------------[*]
void            CloseDevice     (void)
{
    if(gFd)     close(gFd);
    
    LOGI("%s : %s closed!", __FUNCTION__, ADC_DEVICE_NODE);

    gFd = -1;
}

//[*]--------------------------------------------------------------------------------------------------[*]
//
// Power Button ON/OFF ó
//
//[*]--------------------------------------------------------------------------------------------------[*]
int             OpenChannel     (void)
{
    if(gFd < 0)     {
        LOGI("%s : gFd Failed! (gFd < 0) Error!", __FUNCTION__);
        return  -1;     // Can't Open
    }
        
    // ADC Sampling Start
    if(ioctl(gFd, ADC_SET_ENABLE, 1) < 0)    return  -1;

    LOGI("%s : ADC Enabled!", __FUNCTION__);

    return  0;        
}

//[*]--------------------------------------------------------------------------------------------------[*]
int             StopChannel     (void)
{
    if(gFd < 0)     {
        LOGI("%s : gFd Failed! (gFd < 0) Error!", __FUNCTION__);
        return  -1;     // Can't Open
    }
        
    // ADC Sampling Start
    if(ioctl(gFd, ADC_SET_ENABLE, 0) < 0)    return  -1;

    LOGI("%s : ADC Disabled!", __FUNCTION__);

    return  0;        
}

//[*]--------------------------------------------------------------------------------------------------[*]
//
//  ǥõǾ Plot Data ش 
//
//[*]--------------------------------------------------------------------------------------------------[*]
void            SetScale        (unsigned int scale)
{
    if(scale < VOLT_DIV_COUNT)  gScale = scale;
    else                        gScale = 0;
}

//[*]--------------------------------------------------------------------------------------------------[*]
int             GetScale        (void)
{
    return  gScale;
}

//[*]--------------------------------------------------------------------------------------------------[*]
float            GetVoltScale    (void)
{
    return  VOLT_PER_DIV[gScale];
}

//[*]--------------------------------------------------------------------------------------------------[*]
//
//  ǥõǾ Data  
//
//[*]--------------------------------------------------------------------------------------------------[*]
void            SetPeriod       (unsigned int period)
{
    if(period < PERIODS_COUNT)  gPeriod = period;
    else                        gPeriod = 0;
}

//[*]--------------------------------------------------------------------------------------------------[*]
unsigned int    GetPeriod       (void)
{
    return  gPeriod;
}

//[*]--------------------------------------------------------------------------------------------------[*]
unsigned int    GetDataSize     (void)
{
    return  PERIODS[gPeriod];
}

//[*]--------------------------------------------------------------------------------------------------[*]
//
// ȭ鿡 ǥõǾ  
//
//[*]--------------------------------------------------------------------------------------------------[*]
int             GetDeviceData   (unsigned int scale, unsigned int period)
{
    if(gFd < 0)     {
        LOGI("%s : gFd Failed! (gFd < 0) Error!", __FUNCTION__);
        return  -1;     // Can't Open
    }

    SetScale(scale);    SetPeriod(period);

    if(ioctl(gFd, ADC_SET_PERIOD, GetDataSize()) < 0)   {
        LOGI("%s : ADC_SET_PERIOD Failed!", __FUNCTION__);
        return  -1;
    }
        
    if(ioctl(gFd, ADC_REQ_BUFFER, gDataPool) < 0)       {
        LOGI("%s : ADC_REQ_BUFFER Failed!", __FUNCTION__);
        return  -1;
    }

    return  0;
}

//[*]--------------------------------------------------------------------------------------------------[*]
int             GetPlotData     (unsigned int *pData)
{
    float    plot_data, mV_scale;

	int data_pos_offset = 0;
    
    int     i;
    
    data_pos_offset = ((GetDataSize() -1) / (SCREEN_HORIZONTAL_PIXEL -1));
    
    mV_scale        = 2 * 3300 / 0xfff;

    for(i = 0; i < SCREEN_HORIZONTAL_PIXEL; i++)    {

        plot_data = gDataPool[data_pos_offset * i] * mV_scale;
        
        if(plot_data)   plot_data = plot_data * GetVoltScale() / 1000;
		
        if(plot_data > (SCREEN_VERTICAL_PIXEL -1))      plot_data = SCREEN_VERTICAL_PIXEL -1;
        
        pData[i] = (unsigned int)plot_data;
    }
    
    return  0;
}

//[*]--------------------------------------------------------------------------------------------------[*]
//[*]--------------------------------------------------------------------------------------------------[*]

