package com.hardkernel.OdroidOscilloscope;

import com.hardkernel.OdroidOscilloscope.R;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;

public class Scope extends Activity implements Button.OnClickListener {
	
	private ToggleButton mBtnRun;
	private Button mBtnPosDown;
	private Button mBtnPosUp;
	private Button mBtnScaleDec;
	private Button mBtnScaleInc;

	private TextView mTxtVolts;
	private TextView mTxtSec;
	
	private int mFd;
	private static final int NOTI_ADC_LEVEL = 4000;
	private static final int NOTI_ADC_START = 4001;
	private static final int NOTI_ADC_END = 4002;
	private static final int MSG_GET_ADC_READ = 0x1;
	private static final int MSG_GET_ADC_REQ = 0x2;
	private static final int MSG_SAVE_ADC_REQ = 0x4;
	private static final int MSG_GET_ADC_STOP = 0x8;

	private static final int BUFFER_CNT = 602;
	private Message mMessage;
	
	private ScopeDrawWave mScopedraw = null;
	public int [] mVal = new int[BUFFER_CNT];
	
	static String[] mTimeBase = {"6000us", "5400us", "4800us", "4200us", "3800us", "3000us", "2400us", 
																				"1800us", "1200us", "600us"};
	static String[] mAmpScale = {"GND", "2V", "1V", "500mV", "200mV", "100mV", "50mV", "20mV", "10mV"};
	static int mTimeBaseIndex = 9;
	static int mAmpScaleIndex = 3;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
   setContentView(R.layout.main);
    }

	@Override
	protected void onPause() {
		// TODO Auto-generated method stub
		super.onPause();
		mScopedraw.stop();
		
		if(mFd >= 0) 
			closeScope();
		
		mFd = -1;
		
		mMessage = mHandler.obtainMessage();
		mMessage.what = MSG_GET_ADC_STOP;
		mMessage.obj = 0;
		mHandler.sendMessage(mMessage);
	}

	@Override
	protected void onResume() {
		// TODO Auto-generated method stub
	
		super.onResume();
	}

	@Override
	protected void onDestroy() {
		
		super.onDestroy();
	}
	
	Handler mHandler = new Handler() {
		@Override
		public void handleMessage(android.os.Message msg) {
			switch(msg.what) {
				case MSG_GET_ADC_READ:
					int i;
					for(i = 0; i < BUFFER_CNT; i++) {
						if(mVal[i] > NOTI_ADC_LEVEL) {
							if(mVal[i] == NOTI_ADC_START) {
								msg = mHandler.obtainMessage();
								msg.what = MSG_SAVE_ADC_REQ;
								msg.obj = 0;
								mHandler.sendMessage(msg);
							} else if (mVal[i] == NOTI_ADC_END) {
								if(mScopedraw.setData(mVal)) {
									msg = mHandler.obtainMessage();
									msg.what = MSG_GET_ADC_REQ;
									msg.obj = 0;
									mHandler.sendMessage(msg);
									try {
										Thread.sleep(50);
									} catch (InterruptedException e) {
										// TODO Auto-generated catch block
										e.printStackTrace();
									}
								}
							}
						} 
					}
					break;

				case MSG_GET_ADC_REQ:
			
					if(mBtnRun.isChecked()) {
						mVal = readScope();
						msg = mHandler.obtainMessage();
						msg.what = MSG_GET_ADC_READ;
						msg.obj = 0;
						mHandler.sendMessage(msg);
					}
					break;
					
				case MSG_SAVE_ADC_REQ:
					requestADC(mAmpScaleIndex, mTimeBaseIndex);
					break;

				case MSG_GET_ADC_STOP:
				default:;
			}
		};
	};
	
	@Override
	protected void onStart() {
		// TODO Auto-generated method stub
		super.onStart();
		
		setuphklScope();
	}
	
	public void onClick(View v) {
		int btnID;
		btnID = v.getId();
		switch(btnID) {

		case R.id.btnRun :
			if (mFd < 0)
				Toast.makeText(this, "Can not open ADC device!", Toast.LENGTH_SHORT).show();
			else {
				if(mBtnRun.isChecked()) {
					powerOnScope();
					requestADC(mAmpScaleIndex, mTimeBaseIndex);
					mMessage = mHandler.obtainMessage();
					mMessage.what = MSG_GET_ADC_REQ;
					mMessage.obj = 0;
					mHandler.sendMessage(mMessage);
				} else {
					mMessage = mHandler.obtainMessage();
					mMessage.what = MSG_GET_ADC_STOP;
					mMessage.obj = 0;
					mHandler.sendMessage(mMessage);
					powerOffScope();
				}
			}
			break;
		
		case R.id.scaleH_dec :
			if(mTimeBaseIndex < 9)
				mTxtSec.setText(mTimeBase[++mTimeBaseIndex]);
			break;
		
		case R.id.scaleH_inc :
			if(mTimeBaseIndex > 0)
				mTxtSec.setText(mTimeBase[--mTimeBaseIndex]);
			break;

		case R.id.scaleV_dec :
			if(mAmpScaleIndex < 8)
				mTxtVolts.setText(mAmpScale[++mAmpScaleIndex]);
			break;
		
		case R.id.scaleV_inc :
			if(mAmpScaleIndex > 0)
				mTxtVolts.setText(mAmpScale[--mAmpScaleIndex]);
			break;

		default : ;
		}
	}
	
	private void setuphklScope() {

		mTxtVolts = (TextView)findViewById(R.id.volts_div);
		mTxtVolts.setText(mAmpScale[mAmpScaleIndex]);

		mTxtSec = (TextView)findViewById(R.id.sec_div);
		mTxtSec.setText(mTimeBase[mTimeBaseIndex]);

   mBtnRun = (ToggleButton)findViewById(R.id.btnRun);
   mBtnRun.setOnClickListener(this);

		mBtnScaleDec = (Button)findViewById(R.id.scaleH_dec);
   mBtnScaleDec.setOnClickListener(this);

		mBtnScaleInc = (Button)findViewById(R.id.scaleH_inc);
   mBtnScaleInc.setOnClickListener(this);

		mBtnScaleDec = (Button)findViewById(R.id.scaleV_dec);
   mBtnScaleDec.setOnClickListener(this);

		mBtnScaleInc = (Button)findViewById(R.id.scaleV_inc);
   mBtnScaleInc.setOnClickListener(this);
   
		mFd = openScope();
		if (mFd < 0)
			Log.e("Scope", "open adc driver fail!");

		mScopedraw = (ScopeDrawWave)findViewById(R.id.drawWave);
		mScopedraw = new ScopeDrawWave(this, null);
	}
	
	public native int openScope();
	public native int [] readScope();
	public native int requestADC(int h, int v);
	public native void closeScope();
	public native int powerOnScope();
	public native int powerOffScope();

	static {
		System.loadLibrary("Scope");
    }
}
