package com.hardkernel.OdroidOscilloscope;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.util.Log;
import android.view.SurfaceHolder.Callback;

public class ScopeDrawWave extends SurfaceView implements Callback {

	private WaveFormThread plot_thread;
	
	private final int width = 600;
	private final int height = 380;
	
	private static int [] ch1_data = new int[600];
	private static int ch1_pos = 380;
                    
	private Paint ch1_color = new Paint();
	private Paint grid_paint = new Paint(); 
	private Paint cross_paint = new Paint();
	private Paint outline_paint = new Paint();
	
	public ScopeDrawWave(Context context, AttributeSet attrs) {
		 super(context, attrs);
		// TODO Auto-generated constructor stub

		getHolder().addCallback(this);
		
		for(int i = 0; i < width; i++)
		 	ch1_data[i] = ch1_pos;

		plot_thread = new WaveFormThread(getHolder(), this);
		ch1_color.setColor(Color.YELLOW);
		ch1_color.setStrokeWidth(3.0f);
		grid_paint.setColor(Color.rgb(100, 100, 100));
		cross_paint.setColor(Color.rgb(70, 100, 70));
		outline_paint.setColor(Color.GREEN);
	}

	@Override
	public void surfaceChanged(SurfaceHolder arg0, int arg1, int arg2, int arg3) {
		// TODO Auto-generated method stub
		System.out.printf("===> surfaceChanged()\n");

	}

	@Override
	public void surfaceCreated(SurfaceHolder arg0) {
		// TODO Auto-generated method stub
		System.out.printf("===> surfaceCreated()\n");
		plot_thread.setRunning(true);
		plot_thread.start();
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder arg0) {
		// TODO Auto-generated method stub
		boolean retry = true;
		System.out.printf("===> surfaceDestroyed()\n");
		plot_thread.setRunning(false);
		while (retry) {
			try {
				plot_thread.join();
				retry = false;
			}catch(InterruptedException e){ 
			}   
        } 
	}

	@Override
	public void onDraw(Canvas canvas) {
		System.out.printf("===> onDraw()\n");
		PlotPoints(canvas);
	}
	
	public boolean setData(int [] data1) {
    
		int i = 1;
        
		plot_thread.setRunning(false);

		while(i < width)
			ch1_data[i++] = height - data1[i];

		plot_thread.setRunning(true);

		return true;
	}
	
	public void stop() {
		plot_thread.setRunning(false);
		plot_thread.stop();
	}
	
	public void PlotPoints(Canvas canvas) {

		// clear screen
		canvas.drawColor(Color.rgb(20, 20, 20));
			
		// draw grids
		for(int vertical = 1; vertical<10; vertical++) {
			canvas.drawLine(vertical*(width/10)+1, 1, vertical*(width/10)+1, height+1, grid_paint);
        }
        
		for(int horizontal = 1; horizontal<10; horizontal++) {
			canvas.drawLine(1, horizontal*(height/10)+1, width+1, horizontal*(height/10)+1, grid_paint);
        }

		// draw center cross
		canvas.drawLine(0, (height/2)+1, width+1, (height/2)+1, cross_paint);
		canvas.drawLine((width/2)+1, 0, (width/2)+1, height+1, cross_paint);

		// draw outline
		canvas.drawLine(0, 0, (width+1), 0, outline_paint); // top
		canvas.drawLine((width+1), 0, (width+1), (height+1), outline_paint); //right
		canvas.drawLine(0, (height+1), (width+1), (height+1), outline_paint); // bottom
		canvas.drawLine(0, 0, 0, (height+1), outline_paint); //left

		// plot data
		for(int x=0; x < (width-1); x++) {
			canvas.drawLine(x+1, ch1_data[x], x+2, ch1_data[x+1], ch1_color);
        }
    }
}
